@@ -504,6 +504,9 @@ macro(conan_basic_setup)
     if(CONAN_EXPORTED)
         message(STATUS "Conan: called by CMake conan helper")
     endif()
+    if(CONAN_IN_LOCAL_CACHE)
+        message(STATUS "Conan: called inside local cache")
+    endif()
     conan_check_compiler()
     if(NOT ARGUMENTS_NO_OUTPUT_DIRS)
         conan_output_dirs_setup()
@@ -614,6 +617,9 @@ macro(conan_basic_setup)
     if(CONAN_EXPORTED)
         message(STATUS "Conan: called by CMake conan helper")
     endif()
+    if(CONAN_IN_LOCAL_CACHE)
+        message(STATUS "Conan: called inside local cache")
+    endif()
     conan_check_compiler()
     # conan_output_dirs_setup()
     if(NOT ARGUMENTS_TARGETS)
