@@ -6,7 +6,8 @@ from collections import namedtuple
 from conans import tools
 from conans.client.build.autotools_environment import AutoToolsBuildEnvironment
 from conans.client.tools.oss import cpu_count
-from conans.model.ref import ConanFileReference
+from conans.errors import ConanException
+from conans.model.ref import ConanFileReference, PackageReference
 from conans.model.settings import Settings
 from conans.paths import CONANFILE
 from conans.test.build_helpers.cmake_test import ConanFileMock
@@ -15,6 +16,30 @@ from conans.test.utils.conanfile import MockConanfile, MockSettings, MockOptions
 from conans.test.utils.tools import TestClient
 
 
+default_dirs_flags = ["--bindir", "--libdir", "--includedir", "--datarootdir", "--libdir",
+                      "--sbin", "--oldincludedir", "--libexec"]
+
+
+class MockConanfileWithOutput(MockConanfile):
+    def run(self, *args, **kwargs):
+        if self.runner:
+            self.runner(*args, **kwargs)
+
+
+class RunnerMockWithHelp(RunnerMock):
+
+    def __init__(self, return_ok=True, available_args=None):
+        self.command_called = None
+        self.return_ok = return_ok
+        self.available_args = available_args or []
+
+    def __call__(self, command, output=None, win_bash=False, subsystem=None):  # @UnusedVariable
+        if "configure --help" in command:
+            output.write(" ".join(self.available_args))
+        else:
+            return super(RunnerMockWithHelp, self).__call__(command, output, win_bash, subsystem)
+
+
 class AutoToolsConfigureTest(unittest.TestCase):
 
     def _set_deps_info(self, conanfile):
@@ -47,6 +72,16 @@ class AutoToolsConfigureTest(unittest.TestCase):
         be.make()
         self.assertIsNone(conan_file.command)
 
+    def warn_when_no_triplet_test(self):
+        conan_file = ConanFileMock()
+        deps_cpp_info = namedtuple("Deps", "libs, include_paths, lib_paths, defines, cflags, "
+                                   "cppflags, sharedlinkflags, exelinkflags, sysroot")
+        conan_file.deps_cpp_info = deps_cpp_info([], [], [], [], [], [], [], [], "")
+        conan_file.settings = MockSettings({"arch": "UNKNOWN_ARCH", "os": "Linux"})
+        AutoToolsBuildEnvironment(conan_file)
+        self.assertIn("Unknown 'UNKNOWN_ARCH' machine, Conan doesn't know "
+                      "how to translate it to the GNU triplet", conan_file.output)
+
     def test_cppstd(self):
         options = MockOptions({})
         # Valid one for GCC
@@ -504,28 +539,89 @@ class HelloConan(ConanFile):
         ab.install()
         self.assertEquals(runner.command_called, "make install -j%s" % cpu_count())
 
-    def autotools_prefix_test(self):
-        runner = RunnerMock()
-        conanfile = MockConanfile(MockSettings({}), None, runner)
+    def autotools_install_dir_custom_configure_test(self):
+        for flag_to_remove in default_dirs_flags:
+            flags_available = set(default_dirs_flags) - set([flag_to_remove])
+            runner = RunnerMockWithHelp(available_args=flags_available)
+            conanfile = MockConanfileWithOutput(MockSettings({}), None, runner)
+            conanfile.package_folder = "/package_folder"
+            ab = AutoToolsBuildEnvironment(conanfile)
+            ab.configure()
+            self.assertNotIn(flag_to_remove, runner.command_called)
+            for flag_applied in flags_available:
+                self.assertIn(flag_applied, runner.command_called)
+
+    def failing_configure_help_test(self):
+
+        class RunnerMockWithHelpFailing(RunnerMockWithHelp):
+            def __call__(self, command, output=None, win_bash=False, subsystem=None):  # @UnusedVariable
+                if "configure --help" in command:
+                    raise ConanException("Help not available")
+                else:
+                    return super(RunnerMockWithHelp, self).__call__(command, output, win_bash, subsystem)
+
+        runner = RunnerMockWithHelpFailing(available_args=default_dirs_flags)
+        conanfile = MockConanfileWithOutput(MockSettings({}), None, runner)
+        conanfile.package_folder = "/package_folder"
+        ab = AutoToolsBuildEnvironment(conanfile)
+        ab.configure()
+        for flag_applied in default_dirs_flags:
+            self.assertNotIn(flag_applied, runner.command_called)
+        self.assertIn("Error running `configure --help`: Help not available", conanfile.output)
+
+    def autotools_install_dirs_test(self):
+
+        runner = RunnerMockWithHelp(available_args=default_dirs_flags)
+        conanfile = MockConanfileWithOutput(MockSettings({}), None, runner)
         # Package folder is not defined
         ab = AutoToolsBuildEnvironment(conanfile)
         ab.configure()
         self.assertNotIn("--prefix", runner.command_called)
+        self.assertNotIn("--bindir", runner.command_called)
+        self.assertNotIn("--libdir", runner.command_called)
+        self.assertNotIn("--includedir", runner.command_called)
+        self.assertNotIn("--dataroot", runner.command_called)
         # package folder defined
         conanfile.package_folder = "/package_folder"
         ab.configure()
         if platform.system() == "Windows":
-            self.assertIn("./configure --prefix=/package_folder", runner.command_called)
+            self.assertIn("./configure --prefix=/package_folder --bindir=${prefix}/bin "
+                          "--sbin=${prefix}/bin --libexec=${prefix}/bin --libdir=${prefix}/lib "
+                          "--includedir=${prefix}/include --oldincludedir=${prefix}/include "
+                          "--datarootdir=${prefix}/res", runner.command_called)
         else:
-            self.assertIn("./configure '--prefix=/package_folder'", runner.command_called)
+            self.assertIn("./configure '--prefix=/package_folder' '--bindir=${prefix}/bin' "
+                          "'--sbin=${prefix}/bin' '--libexec=${prefix}/bin' '--libdir=${prefix}/lib' "
+                          "'--includedir=${prefix}/include' '--oldincludedir=${prefix}/include' "
+                          "'--datarootdir=${prefix}/res'", runner.command_called)
         # --prefix already used in args
         ab.configure(args=["--prefix=/my_package_folder"])
+        self.assertIn("--prefix=/my_package_folder", runner.command_called)
+        self.assertNotIn("--prefix=/package_folder", runner.command_called)
+        # --bindir, --libdir, --includedir already used in args
+        ab.configure(args=["--bindir=/pf/superbindir", "--libdir=/pf/superlibdir",
+                           "--includedir=/pf/superincludedir"])
+        self.assertNotIn("--bindir=${prefix}/bin", runner.command_called)
+        self.assertNotIn("--libdir=${prefix}/lib", runner.command_called)
+        self.assertNotIn("--includedir=${prefix}/lib", runner.command_called)
         if platform.system() == "Windows":
-            self.assertIn("./configure --prefix=/my_package_folder", runner.command_called)
-            self.assertNotIn("--prefix=/package_folder", runner.command_called)
+            self.assertIn("./configure --bindir=/pf/superbindir --libdir=/pf/superlibdir "
+                          "--includedir=/pf/superincludedir --prefix=/package_folder "
+                          "--sbin=${prefix}/bin --libexec=${prefix}/bin "
+                          "--oldincludedir=${prefix}/include --datarootdir=${prefix}/res",
+                          runner.command_called)
         else:
-            self.assertIn("./configure '--prefix=/my_package_folder'", runner.command_called)
-            self.assertNotIn("'--prefix=/package_folder'", runner.command_called)
+            self.assertIn("./configure '--bindir=/pf/superbindir' '--libdir=/pf/superlibdir' "
+                          "'--includedir=/pf/superincludedir' '--prefix=/package_folder' "
+                          "'--sbin=${prefix}/bin' '--libexec=${prefix}/bin' "
+                          "'--oldincludedir=${prefix}/include' '--datarootdir=${prefix}/res'",
+                          runner.command_called)
+        # opt-out from default installation dirs
+        ab.configure(use_default_install_dirs=False)
+        self.assertIn("--prefix=/package_folder", runner.command_called)
+        self.assertNotIn("--bindir=${prefix}/bin", runner.command_called)
+        self.assertNotIn("--libdir=${prefix}/lib", runner.command_called)
+        self.assertNotIn("--includedir=${prefix}/lib", runner.command_called)
 
     def autotools_configure_vars_test(self):
         from mock import patch
@@ -596,3 +692,86 @@ class HelloConan(ConanFile):
         self.assertFalse(ab.fpic)
         ab.fpic = True
         self.assertIn("-fPIC", ab.vars["CXXFLAGS"])
+
+    @unittest.skipUnless(platform.system() == "Linux", "Requires make")
+    def autotools_real_install_dirs_test(self):
+        body = r"""#include "hello.h"
+#include <iostream>
+using namespace std;
+
+void hello()
+{
+    cout << "Hola Mundo!";
+}
+"""
+        header = """
+#pragma once
+void hello();
+"""
+        main = """
+#include "hello.h"
+
+int main()
+{
+    hello();
+    return 0;
+}
+"""
+        conanfile = """
+from conans import ConanFile, AutoToolsBuildEnvironment, tools
+
+class TestConan(ConanFile):
+    name = "test"
+    version = "1.0"
+    settings = "os", "compiler", "arch", "build_type"
+    exports_sources = "*"
+
+    def build(self):
+        makefile_am = '''
+bin_PROGRAMS = main
+lib_LIBRARIES = libhello.a
+libhello_a_SOURCES = hello.cpp
+main_SOURCES = main.cpp
+main_LDADD = libhello.a
+'''
+        configure_ac = '''
+AC_INIT([main], [1.0], [luism@jfrog.com])
+AM_INIT_AUTOMAKE([-Wall -Werror foreign])
+AC_PROG_CXX
+AC_PROG_RANLIB
+AM_PROG_AR
+AC_CONFIG_FILES([Makefile])
+AC_OUTPUT
+'''
+        tools.save("Makefile.am", makefile_am)
+        tools.save("configure.ac", configure_ac)
+        self.run("aclocal")
+        self.run("autoconf")
+        self.run("automake --add-missing --foreign")
+        autotools = AutoToolsBuildEnvironment(self)
+        autotools.configure()
+        autotools.make()
+        autotools.install()
+        
+    def package_id(self):
+        # easier to have same package_id for the test
+        self.info.header_only()
+"""
+        client = TestClient()
+        client.save({"conanfile.py": conanfile,
+                     "main.cpp": main,
+                     "hello.h": header,
+                     "hello.cpp": body})
+        client.run("create . danimtb/testing")
+        pkg_path = client.client_cache.package(
+            PackageReference.loads(
+                "test/1.0@danimtb/testing:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9"))
+
+        [self.assertIn(folder, os.listdir(pkg_path)) for folder in ["lib", "bin"]]
+
+        new_conanfile = conanfile.replace("autotools.configure()",
+                                          "autotools.configure(args=['--bindir=${prefix}/superbindir', '--libdir=${prefix}/superlibdir'])")
+        client.save({"conanfile.py": new_conanfile})
+        client.run("create . danimtb/testing")
+        [self.assertIn(folder, os.listdir(pkg_path)) for folder in ["superlibdir", "superbindir"]]
+        [self.assertNotIn(folder, os.listdir(pkg_path)) for folder in ["lib", "bin"]]
