@@ -5,16 +5,22 @@ import shutil
 import sys
 import threading
 import uuid
+import errno
+import stat
 from collections import Counter
 from contextlib import contextmanager
 from io import StringIO
+import subprocess
+import unittest
+import tempfile
+import platform
 
 import bottle
 import requests
 import six
 import time
 from mock import Mock
-from six.moves.urllib.parse import urlsplit, urlunsplit
+from six.moves.urllib.parse import urlsplit, urlunsplit, quote
 from webtest.app import TestApp
 
 from conans import __version__ as CLIENT_VERSION, tools
@@ -24,11 +30,13 @@ from conans.client.conan_api import migrate_and_get_client_cache, Conan, get_req
 from conans.client.conan_command_output import CommandOutputer
 from conans.client.conf import MIN_SERVER_COMPATIBLE_VERSION
 from conans.client.output import ConanOutput
+from conans.client.plugin_manager import PluginManager
 from conans.client.remote_registry import RemoteRegistry
 from conans.client.rest.conan_requester import ConanRequester
 from conans.client.rest.uploader_downloader import IterableToFileAdapter
-from conans.client.tools.scm import Git
+from conans.client.tools.scm import Git, SVN
 from conans.client.userio import UserIO
+from conans.client.tools.files import chdir
 from conans.model.version import Version
 from conans.test.server.utils.server_launcher import (TESTING_REMOTE_PRIVATE_USER,
                                                       TESTING_REMOTE_PRIVATE_PASS,
@@ -41,6 +49,8 @@ from conans.util.files import save_files, save, mkdir
 from conans.util.log import logger
 from conans.model.ref import ConanFileReference, PackageReference
 from conans.model.manifest import FileTreeManifest
+from conans.client.tools.win import get_cased_path
+
 
 
 def inc_recipe_manifest_timestamp(client_cache, conan_ref, inc_time):
@@ -282,6 +292,7 @@ class TestBufferConanOutput(ConanOutput):
 
 def create_local_git_repo(files=None, branch=None, submodules=None, folder=None):
     tmp = folder or temp_folder()
+    tmp = get_cased_path(tmp)
     if files:
         save_files(tmp, files)
     git = Git(tmp)
@@ -303,6 +314,60 @@ def create_local_git_repo(files=None, branch=None, submodules=None, folder=None)
     return tmp.replace("\\", "/"), git.get_revision()
 
 
+def handleRemoveReadonly(func, path, exc):  # TODO: May promote to conan tools?
+    # Credit: https://stackoverflow.com/questions/1213706/what-user-do-python-scripts-run-as-in-windows
+    excvalue = exc[1]
+    if func in (os.rmdir, os.remove, os.unlink) and excvalue.errno == errno.EACCES:
+        os.chmod(path, stat.S_IRWXU| stat.S_IRWXG| stat.S_IRWXO) # 0777
+        func(path)
+    else:
+        raise
+
+
+class SVNLocalRepoTestCase(unittest.TestCase):
+    path_with_spaces = True
+
+    def _create_local_svn_repo(self):
+        repo_url = os.path.join(self._tmp_folder, 'repo_server')
+        subprocess.check_output('svnadmin create "{}"'.format(repo_url), shell=True)
+        return SVN.file_protocol + quote(repo_url.replace("\\", "/"), safe='/:')
+
+    def gimme_tmp(self, create=True):
+        tmp = os.path.join(self._tmp_folder, str(uuid.uuid4()))
+        if create:
+            os.makedirs(tmp)
+        return tmp
+
+    def create_project(self, files, rel_project_path=None, commit_msg='default commit message', delete_checkout=True):
+        tmp_dir = self.gimme_tmp()
+        try:
+            rel_project_path = rel_project_path or str(uuid.uuid4())
+            # Do not use SVN class as it is what we will be testing
+            subprocess.check_output('svn co "{url}" "{path}"'.format(url=self.repo_url, path=tmp_dir), shell=True)
+            tmp_project_dir = os.path.join(tmp_dir, rel_project_path)
+            os.makedirs(tmp_project_dir)
+            save_files(tmp_project_dir, files)
+            with chdir(tmp_project_dir):
+                subprocess.check_output("svn add .", shell=True)
+                subprocess.check_output('svn commit -m "{}"'.format(commit_msg), shell=True)
+                rev = subprocess.check_output("svn info --show-item revision", shell=True).decode().strip()
+            project_url = self.repo_url + "/" + quote(rel_project_path.replace("\\", "/"))
+            return project_url, rev
+        finally:
+            if delete_checkout:
+                shutil.rmtree(tmp_dir, ignore_errors=False, onerror=handleRemoveReadonly)
+
+    def run(self, *args, **kwargs):
+        tmp_folder = tempfile.mkdtemp(suffix='_conans')
+        try:
+            self._tmp_folder = os.path.join(tmp_folder, 'path with spaces' if self.path_with_spaces else 'pathwithoutspaces')
+            os.makedirs(self._tmp_folder)
+            self.repo_url = self._create_local_svn_repo()
+            super(SVNLocalRepoTestCase, self).run(*args, **kwargs)
+        finally:
+            shutil.rmtree(tmp_folder, ignore_errors=False, onerror=handleRemoveReadonly)
+
+
 class MockedUserIO(UserIO):
 
     """
@@ -459,10 +524,15 @@ class TestClient(object):
             self.requester = ConanRequester(requester, self.client_cache,
                                             get_request_timeout())
 
+            self.plugin_manager = PluginManager(self.client_cache.plugins_path,
+                                                get_env("CONAN_PLUGINS", list()),
+                                                self.user_io.out)
+
             self.localdb, self.rest_api_client, self.remote_manager = Conan.instance_remote_manager(
                                                             self.requester, self.client_cache,
                                                             self.user_io, self.client_version,
-                                                            self.min_server_compatible_version)
+                                                            self.min_server_compatible_version,
+                                                            self.plugin_manager)
             set_global_instances(output, self.requester)
 
     def init_dynamic_vars(self, user_io=None):
@@ -483,7 +553,7 @@ class TestClient(object):
             # Settings preprocessor
             interactive = not get_env("CONAN_NON_INTERACTIVE", False)
             conan = Conan(self.client_cache, self.user_io, self.runner, self.remote_manager,
-                          interactive=interactive)
+                          self.plugin_manager, interactive=interactive)
         outputer = CommandOutputer(self.user_io, self.client_cache)
         command = Command(conan, self.client_cache, self.user_io, outputer)
         args = shlex.split(command_line)
@@ -503,9 +573,14 @@ class TestClient(object):
                 sys.modules.pop(added, None)
 
         if not ignore_error and error:
-            logger.error(self.user_io.out)
-            print(self.user_io.out)
-            raise Exception("Command failed:\n%s" % command_line)
+            exc_message = "\n{command_header}\n{command}\n{output_header}\n{output}\n{output_footer}\n".format(
+                command_header='{:-^80}'.format(" Command failed: "),
+                output_header='{:-^80}'.format(" Output: "),
+                output_footer='-'*80,
+                command=command_line,
+                output=self.user_io.out
+            )
+            raise Exception(exc_message)
 
         self.all_output += str(self.user_io.out)
         return error
